#
#++
#
#  Copyright Digital Equipment Corporation 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_IFMP.DDL
#
#  ABSTRACT:
#
#	DDL program to decode Ipsilon Flow Management Protocol packets.
#
#  AUTHORS:
#
#      John Rigby
#
#  DATE:
#
#      15 Apr 1997
#
#  MODIFICATION HISTORY:
#
#
INCLUDE ddlinc.ddl

#external declarations
#
DECLARE PST 
	pst_ip_protocols;	#from d_ip4.ddl - relies on d_ip4.ddl convention that header length is in $32
#
#forward declarations
#
DECLARE PST 
	pst_ifmp,		#local - decodes ifmp message
	pst_ifmp_adj_addr,	#local - decodes ifmp adj protocol addr sections
	pst_ifmp_redirect_body,	#local - decodes ifmp redirect message body sections
	;
DECLARE DECODE
	d_ifmp,			#local - to attach to ip_protocols
	d_ifmp_unknown_opcode,	#local - unknown opcode
	d_ifmp_syn,		#local - syn opcode
	d_ifmp_synack,		#local - synack opcode
	d_ifmp_rstack,		#local - rstack opcode
	d_ifmp_ack,		#local - ack opcode
	d_ifmp_redirect,	#local - redirect opcode
	d_ifmp_reclaim,		#local - reclaim opcode
	d_ifmp_adj_addr,	#local - adj protocol address field
	d_ifmp_redirect_0,	#local - redirect flow type 0
	d_ifmp_redirect_1,	#local - redirect flow type 1
	d_ifmp_redirect_2,	#local - redirect flow type 2
	;
DECLARE NAME_TABLE 
	IpProtocols
	;
#
# Constants
#
#    0                   1                   2                   3
#    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |    Version    |    Op Code    |           Checksum            |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                        Sender Instance                        |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                         Peer Instance                         |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                         Peer Identity                         |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                    Peer Next Sequence Number                  |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |         Reserved              |    Reserved   | Max Ack Intvl |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                                                               |
#   ~                          Address List                         ~
#   |                                                               |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
CONSTANT 
	IFMP_version		=  0,
	IFMP_opcode		=  1,
	IFMP_checksum		=  2,
	IFMP_sender_instance	=  4,
	IFMP_peer_instance	=  8,
	IFMP_peer_identity	= 12,
	IFMP_peer_next_seq	= 16,
	IFMP_max_ack_interval	= 23,
	IFMP_address_list	= 24;
#    0                   1                   2                   3
#    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |    Version    |    Op Code    |           Checksum            |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                        Sender Instance                        |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                         Peer Instance                         |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                        Sequence Number                        |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                                                               |
#   ~                          Message Body                         ~
#   |                                                               |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT 
	IFMP_sequence_number	= 12,
	IFMP_message_body	= 16;
#    0                   1                   2                   3
#    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                         Minimum Label                         |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                         Maximum Label                         |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
CONSTANT
	IFMP_minimum_label	= 16,
	IFMP_maximum_label	= 20,
	IFMP_label_range_length = 24;
#    0                   1                   2                   3
#    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |  Error Code   |                  Parameter                    |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#
CONSTANT
	IFMP_error_code		= 16,
	IFMP_error_parameter	= 17,
	IFMP_error_length	= 20;
# For redirect protocol messages one of more flows follow the header:
#    0                   1                   2                   3
#    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |   Flow Type   | Flow ID Length|           Lifetime            |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                             Label                             |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |Version|  IHL  |Type of Service| Time to Live  |   Protocol    |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                         Source Address                        |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |                      Destination Address                      |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
#   |          Source Port          |       Destination Port        |
#   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

CONSTANT
	IFMP_redirect_flow_type		= 0,
	IFMP_redirect_flow_id_length	= 1,
	IFMP_redirect_lifetime		= 2,
	IFMP_redirect_label		= 4,
	IFMP_redirect_flow_id		= 8,
	IFMP_redirect_flow_version	= 8,
	IFMP_redirect_flow_tos		= 9,
	IFMP_redirect_flow_ttl		= 10,
	IFMP_redirect_flow_protocol	= 11,
	IFMP_redirect_flow_source	= 12,
	IFMP_redirect_flow_destination	= 16,
	IFMP_redirect_flow_source_port	= 20,#only present for type 1 flows
	IFMP_redirect_flow_dest_port	= 22;#only present for type 1 flows
#
# IFMP opcodes
#
NAME_LIST ifmpOpcodes {
    TITLE = "Opcode",
    OUT_OF_RANGE = "Undefined in RFC1953"
    } =
	"Syn",#0
	"SynAck",#1
	"RstAck",#2
	"Ack",#3
	"Redirect",#4
	"Reclaim",#5
	"ReclaimAck",#6
	"LabelRange",#7
	"Error",#8
	;

#
# IFMP error codes
#
NAME_LIST ifmpErrorCodes {
    TITLE = "Error Code",
    OUT_OF_RANGE = "Undefined in RFC1953"
    } =
	"Undefined in RFC1953",#0
	"Bad version",#1
	"Bad Flow type",#2
	;

DECODE d_ifmp(101) {
        TITLE = "IFMP",
        ATTACH = pst_ip_protocols,
        PLEVEL = HL_IFMP,
        NEXT = pst_ifmp,
        CONSUME = $32
        } =
        header_checksum( IFMP_checksum, 0, "Verified IFMP checksum" )
        ;
#
# PST: pst_ifmp
#
PST pst_ifmp( byte(IFMP_opcode) ) {
    TITLE = "IFMP opcode",
    DEFAULT = d_ifmp_unknown_opcode
    } = ;

DECODE d_ifmp_unknown_opcode(0) {
    TITLE = "Undecoded",
    PLEVEL = HL_IFMP,
    CONSUME = 2
    } = 
	decimal( byte(IFMP_version), 0, "Version" ),
	list( byte(IFMP_opcode), 0, ifmpOpcodes );

DECODE d_ifmp_syn(0) {
    TITLE = "SYN",
    PLEVEL = HL_IFMP,
    CONSUME = IFMP_address_list + 4,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_adj_addr
    } = 
	decimal( byte(IFMP_version), 0, "Version" ),
	hex(long(IFMP_sender_instance),8,"Sender Instance"),
	hex(long(IFMP_peer_instance),8,"Peer Instance"),
	ip_address(long(IFMP_peer_identity),0,"Peer Identity"){BRIEF},
	decimal(long(IFMP_peer_next_seq),0,"Peer next sequence"),
	decimal(byte(IFMP_max_ack_interval),1,"Maximum Acknowledge Interval"),
	ip_address(long(IFMP_address_list),0,"Address");

PST pst_ifmp_adj_addr(0) {
    DEFAULT = d_ifmp_adj_addr,
    TITLE = ""
    } = ;

DECODE d_ifmp_adj_addr(0) {
    TITLE = "",
    CONSUME = 4,
    NEXT = pst_ifmp_adj_addr
    } = ip_address (long(0),0,"Address");

DECODE d_ifmp_synack(1) {
    TITLE = "SynAck",
    PLEVEL = HL_IFMP,
    CONSUME = IFMP_address_list + 4,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_adj_addr
    } = REF d_ifmp_syn;
DECODE d_ifmp_rstack(2) {
    TITLE = "RstAck",
    PLEVEL = HL_IFMP,
    CONSUME = IFMP_address_list + 4,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_adj_addr
    } = REF d_ifmp_syn;
DECODE d_ifmp_ack(3) {
    TITLE = "Ack",
    PLEVEL = HL_IFMP,
    CONSUME = IFMP_address_list + 4,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_adj_addr
    } = REF d_ifmp_syn;

DECODE d_ifmp_redirect(4) {
    TITLE = "Redirect",
    PLEVEL = HL_IFMP,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_redirect_body,
    CONSUME = IFMP_message_body
    } = 
	decimal( byte(IFMP_version), 0, "Version" ),
	hex(long(IFMP_sender_instance),8,"Sender Instance"),
	hex(long(IFMP_peer_instance),8,"Peer Instance"),
	decimal(long(IFMP_sequence_number),0,"Sequence number"){BRIEF};

PST pst_ifmp_redirect_body( byte(0) ) {
    TITLE = "Flow"
    } =
	d_ifmp_redirect_0,
	d_ifmp_redirect_1,
	d_ifmp_redirect_2;

DECODE d_ifmp_redirect_0(0) {
    TITLE = "Type 0",
    PLEVEL = HL_IFMP,
    NEXT = pst_ifmp_redirect_body,
    CONSUME = $32
    } =
# Compute the length of this block
	$32 = byte(IFMP_redirect_flow_id_length),
	$32 = $32*4+IFMP_redirect_flow_id,
# now display the contents
	decimal(word(IFMP_redirect_lifetime),0,"Redirect lifetime") {CONDITION = word(IFMP_redirect_lifetime)},
	hex(long(IFMP_redirect_label),8,"Redirect label"){BRIEF};

DECODE d_ifmp_redirect_1(1) {
    TITLE = "Type 1",
    PLEVEL = HL_IFMP,
    NEXT = pst_ifmp_redirect_body,
    CONSUME = $32
    } =
# Compute the length of this block
	$32 = byte(IFMP_redirect_flow_id_length),
	$32 = $32*4+IFMP_redirect_flow_id,
# now display the contents
	decimal(word(IFMP_redirect_lifetime),0,"Redirect lifetime") {CONDITION = word(IFMP_redirect_lifetime)},
	hex(long(IFMP_redirect_label),8,"Redirect label"){BRIEF},
# now the flow type 1 identifier
	hex(byte(IFMP_redirect_flow_version),2,"IP version/length"),
	hex(byte(IFMP_redirect_flow_tos),2,"Type of Service"),
	decimal(byte(IFMP_redirect_flow_ttl),0,"Time to live"){BRIEF},
	name( "Protocol", IpProtocols, byte(IFMP_redirect_flow_protocol) ){BRIEF},
	ip_address(long(IFMP_redirect_flow_source),0,"Source Address"){BRIEF},
	ip_address(long(IFMP_redirect_flow_destination),0,"Destination Address"){BRIEF},
	hex(word(IFMP_redirect_flow_source_port),4,"Source Port"){BRIEF},
	hex(word(IFMP_redirect_flow_dest_port),4,"Destination Port"){BRIEF};

DECODE d_ifmp_redirect_2(2) {
    TITLE = "Type 2",
    PLEVEL = HL_IFMP,
    NEXT = pst_ifmp_redirect_body,
    CONSUME = $32
    } =
# Compute the length of this block
	$32 = byte(IFMP_redirect_flow_id_length),
	$32 = $32*4+IFMP_redirect_flow_id,
# now display the contents
	decimal(word(IFMP_redirect_lifetime),0,"Redirect lifetime") {CONDITION = word(IFMP_redirect_lifetime)},
	hex(long(IFMP_redirect_label),8,"Redirect label"){BRIEF},
# now the flow type 2 identifier
	hex(byte(IFMP_redirect_flow_version),2,"IP version/length"),
	decimal(byte(IFMP_redirect_flow_ttl),0,"Time to live"){BRIEF},
	ip_address(long(IFMP_redirect_flow_source),0,"Source Address"){BRIEF},
	ip_address(long(IFMP_redirect_flow_destination),0,"Destination Address"){BRIEF};

DECODE d_ifmp_reclaim(5) {
    TITLE = "Reclaim",
    PLEVEL = HL_IFMP,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_redirect_body,
    CONSUME = IFMP_message_body
    } = REF d_ifmp_redirect;

DECODE d_ifmp_reclaimack(6) {
    TITLE = "Reclaim Acknowledge",
    BRIEF_TITLE = "RecAck",
    PLEVEL = HL_IFMP,
    ATTACH = pst_ifmp,
    NEXT = pst_ifmp_redirect_body,
    CONSUME = IFMP_message_body
    } = REF d_ifmp_redirect;

DECODE d_ifmp_label_range(7) {
    TITLE = "Label Range",
    BRIEF_TITLE = "LabRng",
    PLEVEL = HL_IFMP,
    ATTACH = pst_ifmp,
    CONSUME = IFMP_label_range_length
    } = 
	decimal( byte(IFMP_version), 0, "Version" ),
	hex(long(IFMP_sender_instance),8,"Sender Instance"),
	hex(long(IFMP_peer_instance),8,"Peer Instance"),
	decimal(long(IFMP_sequence_number),0,"Sequence number"){BRIEF},
	hex(long(IFMP_minimum_label),8,"Minimum label"){BRIEF},
	hex(long(IFMP_maximum_label),8,"Maximum label"){BRIEF}
	;

DECODE d_ifmp_error(8) {
    TITLE = "Error",
    PLEVEL = HL_IFMP,
    ATTACH = pst_ifmp,
    CONSUME = IFMP_error_length
    } = 
	decimal( byte(IFMP_version), 0, "Version" ),
	hex(long(IFMP_sender_instance),8,"Sender Instance"),
	hex(long(IFMP_peer_instance),8,"Peer Instance"),
	decimal(long(IFMP_sequence_number),0,"Sequence number"){BRIEF},
	list(byte(IFMP_error_code),0,ifmpErrorCodes){BRIEF},
	hex(b3yte(IFMP_error_parameter),6,"Parameter"){BRIEF}
	;

